
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <dos/dos.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>

#include <clib/AMarquee_protos.h>
#include <devices/inputevent.h>

#include <pragmas/AMarquee_pragmas.h>

struct Library * AMarqueeBase = NULL;
struct QSession * session     = NULL;
FILE * fTest = NULL;


/* Takes user input until blank line is typed */
void ProcessDebugCommands(struct QSession * session)
{
  while(1)
  {
   char debugline[500] = "\0\0\0\0\0\0\0\0";
   char * keyword, * data;
   ULONG dataLen = 0L;
   LONG res;
   
   printf("Enter your debug command now: "); fflush(stdout);
   gets(debugline);
  
   keyword = &debugline[2];
   if (data = strchr(keyword,'='))
   {
     *data = '\0';  /* terminate the keyword */
     data++;
     dataLen = strlen(data)+1;
   }
   switch((int)(debugline[0]))
   {
     case '\0': res=QGo(session,0L);                           break;
     case 'A':  res=QSetMessageAccessOp(session, keyword, -1); break;
     case 'm':  res=QMessageOp(session, keyword, data, dataLen); break;
     case 'a':  res=QSetAccessOp(session, keyword);          break;
     case 's':  res=QSetOp(session, keyword, data, dataLen); break;
     case 'S':  res=QStreamOp(session, keyword, data, dataLen); break;
     case 'r':  res=QRenameOp(session, keyword, data);       break;
     case 'D':  res=QDebugOp(session, keyword);              break;
     case 'g':  res=QGetOp(session, keyword, -1);            break;
     case 'd':  res=QDeleteOp(session, keyword);             break;
     case 'i':  res=QInfoOp(session);                        break;
     case 'c':  res=QSubscribeOp(session, keyword, -1);      break;
     case 'k':  res=QClearSubscriptionsOp(session,atoi(keyword)); break;
     case 'p':  res=QPingOp(session);                        break;
     default:   printf("Command code %c was not recognized.\n",debugline[0]); break;
   }
   printf("(Op result was %i)\n",res);
   if (debugline[0] == '\0') return;
  }
}


void CleanExit(void)
{
printf("\nCleaning up...\n");
  if (fTest) fclose(fTest);
  if (session)      QFreeSession(session);        /* This MUST be done before we close the library! */
  if (AMarqueeBase) CloseLibrary(AMarqueeBase);  
  printf("All done.\n");
}

/* Main program */
int main(int argc, char ** argv)
{
  char * connectTo, * progName;
  int port;
  struct InputEvent ie;

  fTest = fopen("ie.bin","r");    
  if (fTest == NULL) 
  {
    printf("Couldn't open test file ie.bin\n");
    exit(0);
  }
  
  printf("Usage Note:  AMarqueeDebug [hostname=localhost] [myname=ARemote] [port=20000]\n");  
  atexit(CleanExit);
  
  connectTo = (argc>1) ? argv[1] : "localhost";
  progName  = (argc>2) ? argv[2] : "ARemote";
  port      = (argc>3) ? atoi(argv[3]) : 20000;

  if ((AMarqueeBase = OpenLibrary("amarquee.library",43L)) == NULL)
  {
    printf("Couldn't open amarquee.library v43+!\n");
    exit(RETURN_ERROR);
  }
  printf("Connecting to %s:%i\n",connectTo, port);

  if ((session = QNewSession(connectTo, port, progName)) == NULL)
  {
    printf("Couldn't connect to server %s:%i\n",connectTo, port);
    exit(RETURN_WARN);
  }

  printf("Connected to server %s:%i\n",connectTo, port);

  while(fread(&ie, 1, sizeof(struct InputEvent), fTest) == sizeof(struct InputEvent))
  {
    QStreamOp(session, "e", &ie, sizeof(struct InputEvent));
    QGo(session, 0L);
    Delay(1);
  }
  /* CleanExit() called here! */
}
